/* -*- Mode: Vala; indent-tabs-mode: nil; c-basic-offset: 2; tab-width: 2 -*- */
/*
    Multiplication Puzzle
    For copyright information, see AUTHORS.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using GLib;

extern unowned Resource resources_get_resource();

public class GMultApp : Gtk.Application
{
  GtkMult main_window = null;

  const OptionEntry[] options = {
    {"version", 0, 0, OptionArg.NONE, null, N_("Show version"), null},
    {null}
  };

  const ActionEntry[] actions = {
    {"new-game", new_game},
    {"hint", show_hint},
    {"about", about},
    {"quit", quit},
  };

  public GMultApp()
  {
    Object(application_id: "net.launchpad.gmult");
    add_main_option_entries(options);
  }

  public override int handle_local_options(VariantDict options)
  {
    if (options.contains("version")) {
      print("%s\n", Config.VERSION);
      return 0;
    }
    return -1;
  }

  public override void startup()
  {
    base.startup();

    add_action_entries(actions, this);
    set_accels_for_action("app.new-game", {"<Primary>n"});
    set_accels_for_action("app.hint", {"<Primary>h"});
    set_accels_for_action("app.quit", {"<Primary>q"});
  }

  public override void activate()
  {
    base.activate();

    if (main_window != null)
      main_window.present_with_time(Gtk.get_current_event_time());
    else {
      main_window = new GtkMult(this);
      main_window.destroy.connect(() => {this.main_window = null;});
      add_window(main_window);
      main_window.show_all();
      Gdk.notify_startup_complete();
    }
  }

  void new_game()
  {
    main_window.new_puzzle();
  }

  void show_hint()
  {
    main_window.show_hint();
  }

  void about()
  {
    const string[] authors = {"Michael Terry <mike@mterry.name>",
                              null};
    const string[] artists = {"Eugenia Loli-Queru <eloli@hotmail.com>",
                              "Michael Terry <mike@mterry.name>",
                              null};

    Gtk.show_about_dialog(main_window,
                          "authors", authors,
                          "artists", artists,
                          "license-type", Gtk.License.GPL_3_0,
                          "logo-icon-name", "net.launchpad.gmult",
                          "translator-credits", _("translator-credits"),
                          "version", Config.VERSION,
                          "website", "https://launchpad.net/gmult");
  }
}

int main(string[] args)
{
  Intl.textdomain(Config.GETTEXT_PACKAGE);
  Intl.bindtextdomain(Config.GETTEXT_PACKAGE, Config.LOCALE_DIR);
  Intl.bind_textdomain_codeset(Config.GETTEXT_PACKAGE, "UTF-8");

  resources_get_resource()._register();

  Environment.set_prgname("net.launchpad.gmult");
  Environment.set_application_name(_("Multiplication Puzzle"));

  Gtk.Window.set_default_icon_name("net.launchpad.gmult");

  return new GMultApp().run(args);
}
